/*
    (C) 1995-96 AROS - The Amiga Research OS
    $Id: matchfirst.c,v 1.1.1.1 2005/03/15 15:54:40 laire Exp $

    Desc:
    Lang: english
*/
#include "dos_intern.h"
#include <proto/exec.h>
#include <exec/memory.h>
#include <exec/types.h>
#include <dos/dos.h>

static inline
const char *myindex(const char *str, int c)
{
    while (*str != (char) c)
    {
	if (!(*str++))
	{
	    return NULL;
	}
    }
    return str;
}

/*****************************************************************************

    NAME */
#include <dos/dosasl.h>
#include <proto/dos.h>

	AROS_LH2(LONG, MatchFirst,

/*  SYNOPSIS */
	AROS_LHA(STRPTR             , pat, D1),
	AROS_LHA(struct AnchorPath *, AP , D2),

/*  LOCATION */
	struct DosLibrary *, DOSBase, 137, Dos)

/*  FUNCTION

	Searches for the first file or directory that matches a given pattern.
	MatchFirst() initializes the AnchorPath structure for you but you
	must initilize the following fields: ap_Flags, ap_Strlen, ap_BreakBits
	and ap_FoundBreak. The first call to MatchFirst() also passes you
	the first matching file which you can examine in ap_Info and the directory
	the files is in in ap_Current->an_Lock. After the first call to
        MatchFirst() call MatchNext().
	The search begins whereever the current directory is set to. See
	CurrentDir();
	For more info on patterns see ParsePattern().

    INPUTS
        pat  - pattern to search for
	AP   - pointer to (initilized) AnchorPath structure
	
    RESULT
	0     = success
	other = DOS error code

    NOTES

    EXAMPLE

    BUGS

    SEE ALSO
	MatchNext(), MatchEnd(), ParsePattern(), Examine(), CurrentDir()
	<dos/dosasl.h>

    INTERNALS

    HISTORY
        09-11-2000  stegerg, rewritten from scratch.
	29-04-97    bergers, initial revision
	27-11-96    digulla automatically created from
			    dos_lib.fd and clib/dos_protos.h

*****************************************************************************/
{
    AROS_LIBFUNC_INIT
    AROS_LIBBASE_EXT_DECL(struct DosLibrary *,DOSBase)

    struct AChain 	*ac;
    LONG 		error;

    DEBUG_MATCH(dprintf("MatchFirst: pat = \"%s\", anchor = $%lx\n", pat, (ULONG) AP));

    FAILFORTASKNP(return ERROR_NOT_IMPLEMENTED;)

    if ((ULONG) AP & 3)
    {
	DEBUG_WARNINGS(ktprintf("MatchEnd: BUG, AnchorPath 0x%lx isn't aligned by 4!\n", (ULONG) AP));

	/* Special implementation note: we really don't care about alignment! - Piru */
	//return ERROR_ ???;
    }

    AP->ap_Base   = AP->ap_Current = NULL;
    AP->ap_Flags &= ~(APF_ITSWILD | APF_NOMEMERR);

    if ((UWORD) AP->ap_Strlen & APSF_EXTENDED)
    {
	/* ap_Extended is valid, mask out the APSF_EXTENDED flag
	*/
	AP->ap_Strlen &= (UWORD) ~APSF_EXTENDED;
    }
    else
    {
	/* no APSF_EXTENDED not set, no extensions
	 */
	AP->ap_Extended = 0;
    }

    SetIoErr(0);

    if ((!Stricmp(pat, "*") ||
         !Stricmp(pat, "CONSOLE:") ||
         !Stricmp(pat, "NIL:")) ||
        ((myindex(pat, ':') && !IsFileSystem(pat))))
    {
	error = IoErr();

#warning FIXME: Is this code sane? Why was it here in the first place?
	if (error == 0 || error == ERROR_ACTION_NOT_KNOWN)
	{
	    ULONG len = strlen(pat) + 1;

	    if (len <= sizeof(AP->ap_Base->an_Info.fib_FileName) &&
	        (!AP->ap_Strlen || len <= AP->ap_Strlen))
	    {
		AP->ap_Base = AP->ap_Current = Match_AllocAChain(1, DOSBase);
		if (AP->ap_Base)
		{
		    AP->ap_Base->an_Lock = DupLock(((struct Process *) FindTask(NULL))->pr_CurrentDir);
		    AP->ap_Base->an_Flags = DDF_Completed | DDF_Single;

		    memcpy(AP->ap_Info.fib_FileName, pat, len);
		    if (AP->ap_Strlen)
			memcpy(AP->ap_Buf, pat, len);
		    AP->ap_Info.fib_EntryType    = -1;
		    AP->ap_Info.fib_DirEntryType = -1;

		    error = 0;
		}
		else
		    error = ERROR_NO_FREE_STORE;
	    }
	    else
		error = ERROR_BUFFER_OVERFLOW;
	}
    }
    else
    {
	error = Match_BuildAChainList(pat, AP, &ac, DOSBase);
	if (error == 0)
	{
	    AP->ap_Base = AP->ap_Current = ac;

	    error = MatchNext(AP);

	    /* ap_Current->an_Lock changed since last MatchNext call
	     */
	    AP->ap_Flags |= APF_DirChanged;

	} /* if (error == 0) */
    }

    SetIoErr(error);

    DEBUG_MATCH(dprintf("MatchFirst: err = %ld, name = \"%s\"\n", error, error ? "" : AP->ap_Info.fib_FileName));

    return error;

    AROS_LIBFUNC_EXIT

} /* MatchFirst */
